class AR_Game
 {
  private:

   double start_time;

   void init()
    {
	 // Allegro init
     if(!al_init()) { AR_Util::errorBox(NULL,"Failed to initialize allegro!"); exit(1); }

     // Read/Create config file
     config();

     // Install needed allegro components
     al_install_keyboard();
     al_install_mouse();
     al_init_image_addon();
     al_init_font_addon();
     al_init_primitives_addon();
     al_init_acodec_addon();

     // Allegro display init
     display = new AR_Display(screen_width,screen_height,fullscreen);
     display->setMaxFPS(60);
     al_hide_mouse_cursor(display->getDisplay());

     // Audio init
     audio = new AR_Audio(display);

     // Keyboard init
     keyboard = new AR_Keyboard();

     // Font init
     font = new AR_Font("data/font.tga",display);
     
     // Timer init (stopped at start)
     timer = new AR_Timer();

     // Random seed init
     srand(time(NULL));
    }

   void config()
    {    
	 if(!readConfig(AR::configFile())) writeConfig(AR::configFile());

	 // Set config values
	 if(screen_resolution == AR::RES_640x480)  { screen_width = 640;  screen_height = 480; }
	 if(screen_resolution == AR::RES_800x600)  { screen_width = 800;  screen_height = 600; }
	 if(screen_resolution == AR::RES_1024x768) { screen_width = 1024; screen_height = 768; }
	 if(screen_resolution == AR::RES_1152x864) { screen_width = 1152; screen_height = 864; }
	}

   bool readConfig(string filename)
    {
	 ifstream in;
	 string aux;

	 in.open(filename.c_str());
	 if(!in.is_open()) { in.close(); return false; }
     while(in >> aux)
	  {
	   if(aux == "[screen_resolution]") in >> screen_resolution;
	   if(aux == "[fullscreen]")        in >> fullscreen;
	   if(aux == "[debug]")             in >> debug;
	   if(aux == "[key_left]")          in >> key_left;
	  }
	 in.close();
	 return true;
	}

   bool writeConfig(string filename)
    {
	 ofstream out;

     out.open(filename.c_str());
     if(!out.is_open()) { out.close(); return false; }
	 out << "[screen_resolution]" << endl << screen_resolution << endl;
	 out << "[fullscreen]"        << endl << fullscreen << endl;
	 out << "[key_left]"          << endl << key_left   << endl;
	 out << "[key_right]"         << endl << key_right  << endl;
	 out << "[key_up]"            << endl << key_up     << endl;
	 out << "[key_down]"          << endl << key_down   << endl;
	 out << "[key_fire1]"         << endl << key_fire1  << endl;
	 out << "[key_fire2]"         << endl << key_fire2  << endl;
	 out << "[key_fire3]"         << endl << key_fire3;
	 out.close();
	 return true;
	}

  protected:

   int    action;
   int    score;
   double fps;
   bool   show_debug;
   AR_Display  *display;
   AR_Keyboard *keyboard;
   AR_Font     *font;
   AR_Timer    *timer;
   AR_Audio    *audio;

   // Game/Menu Keys
   int key_left,key_right,key_up,key_down;
   int key_fire1,key_fire2,key_fire3,key_esc;
   int key_select,key_space;
   int key_debug;

   // Config variables
   int screen_resolution,screen_width,screen_height; 
   bool fullscreen;
   bool debug;

   void drawToDisplay()
    {
	 al_flip_display();
	}

   void waitFPS()
    {
     al_rest( (1.0 / (double)(display->getMaxFPS())) - (al_get_time() - start_time) );
     fps = 1.0 / (al_get_time() - start_time);
	}

   void getStartTime() { start_time = al_get_time(); }

   void showDebug() { } // Redefine this method

  public:

   AR_Game(int screen_resolution, bool fullscreen)
    {
     // Set default configuration values
	 this->screen_resolution = screen_resolution;
	 this->fullscreen        = fullscreen;
	 debug                   = false;

     // Set default game/menu keys
     key_left   = ALLEGRO_KEY_LEFT;
     key_right  = ALLEGRO_KEY_RIGHT;
     key_up     = ALLEGRO_KEY_UP;
     key_down   = ALLEGRO_KEY_DOWN;
     key_fire1  = ALLEGRO_KEY_Z;
     key_fire2  = ALLEGRO_KEY_X;
     key_fire3  = ALLEGRO_KEY_C;
     key_esc    = ALLEGRO_KEY_ESCAPE;
     key_select = ALLEGRO_KEY_ENTER;
     key_space  = ALLEGRO_KEY_SPACE;
     key_debug  = ALLEGRO_KEY_F8;
     
     // Init all
	 action     = AR::NONE;
	 show_debug = false;
	 score      = 0;
	 init();
	}

   ~AR_Game() { }

   int getScreenWidth()  { return screen_width; }
   
   int getScreenHeight() { return screen_height; }

   void toggleDebug() { if(debug) show_debug = !show_debug; }

   void gameInit() { } // Redefine this method

   void gameRun()  { } // Redefine this method

   int getKeyLeft()   { return key_left;   }

   int getKeyRight()  { return key_right;  }

   int getKeyUp()     { return key_up;     }

   int getKeyDown()   { return key_down;   }
   
   int getKeyFire1()  { return key_fire1;  }
   
   int getKeyFire2()  { return key_fire2;  }
   
   int getKeyFire3()  { return key_fire3;  }
   
   int getKeySelect() { return key_select; }
   
   int getKeySpace()  { return key_space;  }

   int getKeyEsc()    { return key_esc;    }
   
   int getKeyDebug()  { return key_debug;  }

   AR_Keyboard *getKeyboard() { return keyboard; }

 };
