class AR_Map : public AR_Bitmap
 {
  protected:

   int parallax_ratio;             // Parallax ratio: 1 max speed, 2 slow, 4 more slow ...
   float *scroll_x, *scroll_y;     // Scroll synchrony variables
   int tile_width,tile_height;     // Tiles size
   int map_rows,map_columns;       // Number of rows and columns in the map file
   int bitmap_rows,bitmap_columns; // Number of rows and columns in the bitmap file
   int total_tiles;                // Number of total tiles
   ALLEGRO_BITMAP **tiles;         // Map tiles
   int *map;                       // Map tiles indexes

   void reset()
    {
	 // Reset variables to default
	 AR_Bitmap::reset();
	}

  public:

   AR_Map(string filename, AR_Display *display, int parallax_ratio = 1) : AR_Bitmap(filename,display)
    {
	 reset();
	 this->parallax_ratio = parallax_ratio;
     this->x = x;
     this->y = y;
     scroll_x = scroll_y = NULL;
     loadData(filename);
     loadTiles(filename);
    }

   ~AR_Map() { }

   void loadData(string filename)
    {
	 string   aux;
     ifstream fd;

     filename = AR_Util::cutExtension(filename);
     filename += ".map";
     fd.open(filename.c_str());
     if(!fd.is_open()) { AR_Util::errorBox(display->getDisplay(),string("File '") + string(filename) + string("' not found!")); exit(1); }
     fd >> map_rows >> aux >> map_columns >> aux >> tile_width >> aux >> tile_height >> aux >> bitmap_rows >> aux >> bitmap_columns >> aux >> total_tiles >> aux >> aux;

     // Load tile indexes    
     map = (int *)malloc(sizeof(int)*(map_rows*map_columns));
     for(int i=0;i<map_rows;i++)
      for(int j=0;j<map_columns;j++)
        fd >> map[j+map_columns*i];
     fd.close();

    }

   void loadTiles(string filename)
    {
	 int count;
     string str;

     count = 0;
     tiles = (ALLEGRO_BITMAP **)malloc(sizeof(ALLEGRO_BITMAP *)*total_tiles);
     for(int i=0;i<bitmap_rows;i++)
      for(int j=0;j<bitmap_columns;j++)
       {
		if(count<total_tiles)
		 {
          tiles[j+bitmap_columns*i] = al_create_bitmap(tile_width,tile_height);
          al_set_target_bitmap(tiles[j+bitmap_columns*i]);
          al_draw_bitmap_region(bitmap,tile_width*j,tile_height*i,tile_width,tile_height,0,0,0);
	     }
        count++;
       }
     al_set_target_bitmap(al_get_backbuffer(display->getDisplay()));
     al_destroy_bitmap(bitmap); // Destroy bitmap loaded in class AR_Bitmap     
    }

   void debug()
    {
	 float scroll_x, scroll_y;

	 if(!enable) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;
     for(int x=0;x<map_columns;x++)
      for(int y=0;y<map_rows;y++)
       al_draw_pixel(x*tile_width-scroll_x/(float)(parallax_ratio),y*tile_height-scroll_y/(float)(parallax_ratio),al_map_rgb(255,255,0));
	}

   void run() { if(!enable) return; }

   void draw()
    {
	 float scroll_x, scroll_y, xx, yy;
	 if(!enable) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;
     for(int x=0;x<map_columns;x++)
      for(int y=0;y<map_rows;y++)
       {
        xx = x*tile_width-scroll_x/(float)(parallax_ratio);
        yy = y*tile_height-scroll_y/(float)(parallax_ratio);
        if(xx + tile_width > 0 && xx - tile_width < display->getWidth())
         if(yy + tile_width > 0 && yy - tile_width < display->getHeight())
	      al_draw_tinted_bitmap(tiles[map[x+map_columns*y]],
	                            al_map_rgba(alpha * enable_tint_r,
	                                        alpha * enable_tint_g,
	                                        alpha * enable_tint_b,
	                                        alpha),xx,yy,flip_horizontal | flip_vertical);
	   }
    }

   void setScrollXP(float *scroll_x) { this->scroll_x = scroll_x; }
   
   void setScrollYP(float *scroll_y) { this->scroll_y = scroll_y; }
 
   int getMapRows()       { return map_rows;       }

   int getMapColumns()    { return map_columns;    }

   int getBitmapRows()    { return bitmap_rows;    }

   int getBitmapColumns() { return bitmap_columns; }

   int getTileWidth()     { return tile_width;     }

   int getTileHeight()    { return tile_height;    }

   int getMapWidth()  { return map_columns * tile_width; }

   int getMapHeight() { return map_rows * tile_height;   }

 };
