class Cannon : public AR_Sprite
 {
  protected:

   bool  can_shoot;
   bool  enable_targetting;
   int   bullets_fired;
   int   total_bullets;
   int   reload_count,reload_time;
   float rotation_speed;
   float friction_x,friction_y;
   float acceleration_x,acceleration_y;
   float gravity_x,gravity_y;
   float power;
   int   bullet_a_time,bullet_frame;
   int   bullet_damage;
   int   hull_frame,aim_frame;
   int   targetting_range;
   int   weapon;
   Bullet **bullets;
   AR_Sprite *targetting;
   AR_Sample *sample_shoot;

  public:

   Cannon(string filename, AR_Display *display, int total_bullets, AR_Sprite *p = NULL) : AR_Sprite(filename,display,0,0,p)
    {
     this->total_bullets = total_bullets;
     sample_shoot = NULL;
	 bullets = (Bullet **)malloc(sizeof(Bullet *)*total_bullets);
	 for(int i=0;i<total_bullets;i++) bullets[i] = new Bullet("",display,(Bullet *)this);
	 targetting = new AR_Sprite("",display,0,0,p);
     reset();
	}

   ~Cannon() { }

   void shoot()
    {
	 int max;

     if(weapon == AR::NO_CANNON) return;
     if(can_shoot)
      {
	   can_shoot = false; reload_count = 0;
	   max = bullet_a_time;
	   for(int i=0;i<total_bullets;i++)
        if(bullets[i]->getAction() == AR::NONE)
         {
	      bullets[i]->shoot(getX(),getY(),getAngle(),power,life,friction_x,friction_y,acceleration_x,acceleration_y,gravity_x,gravity_y);
	      bullets[i]->setCurrentFrame(bullet_frame);
	      bullets[i]->setDamage(bullet_damage);
	      bullets_fired++;
	      can_shoot    = false;
	      reload_count = 0;
	      if(max==1) break; else max--;
         }
       if(sample_shoot) sample_shoot->play();
	  }
	}

   void run()
    {
	 AR_Sprite::run();
	 for(int i=0;i<total_bullets;i++) bullets[i]->run();
     if(!can_shoot) { reload_count++; if(reload_count == reload_time) can_shoot = true; }
	}

   void draw()
    {
	 int aux;

	 AR_Sprite::draw();
     if(can_shoot) drawTargetting(this->getX(),this->getY(),power,angle,friction_x,friction_y,acceleration_x,acceleration_y);
     for(int i=0;i<total_bullets;i++) bullets[i]->draw();
     aux = current_frame;
     setCurrentFrame(hull_frame);
     AR_Sprite::draw();
     setCurrentFrame(aux);
	}

   void drawTargetting(float x, float y, float power, float angle, float friction_x, float friction_y, float acceleration_x, float acceleration_y)
    {
	 float speed_x,speed_y;
	 float scroll_x,scroll_y;

     if(!enable_targetting)      return;
     if(weapon == AR::NO_CANNON) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;

	 speed_x = power * (cos(AR_Util::radians(angle)));
	 speed_y = power * (-sin(AR_Util::radians(angle)));
     for(int i=0;i<targetting_range;i++)
      {
       x+=speed_x;
       y-=speed_y;
       speed_x+=this->gravity_x;
       speed_y+=this->gravity_y;
       speed_x-=friction_x * speed_x;
       speed_y-=friction_y * speed_y;
       speed_x+=acceleration_x;
       speed_y+=acceleration_y;

       targetting->setCurrentFrame(aim_frame);
       targetting->setAlpha(targetting_range-i);
       targetting->setX(x-scroll_x);
       targetting->setY(y-scroll_y);
       targetting->setAngle(angle);
       if(i%(targetting_range>>5) == 0) targetting->draw();
      }
    }

   void reset()
    {
	 // Reset variables to default
	 AR_Sprite::reset();
	 bullets_fired     = 0;
     can_shoot         = true;
     enable_targetting = false;
     reload_count      = 0;
     reload_time       = 10;
     power             = 1;
     rotation_speed    = 0.4;
     friction_x        =    friction_y     = 0;
     acceleration_x    =    acceleration_y = 0;
     gravity_x         = 0; gravity_y      = -0.0098;
     bullet_frame      = 0;
     bullet_a_time     = 1;
     hull_frame        = 0;
     aim_frame         = 0;
     move_action       = AR::NONE;
     weapon            = AR::NO_CANNON;
	 setCenterX(width>>1);
	 setCenterY(height>>1);
	 targetting->setCenterX(targetting->getWidth()>>1);
	 targetting->setCenterY(targetting->getHeight()>>1);
	 targetting_range = 32;
     for(int i=0;i<total_bullets;i++) bullets[i]->reset();
    }

   void restart()
    {
	 setAngle(270);
	 bullets_fired = 0;
	 for(int i=0;i<total_bullets;i++) bullets[i]->reset();
	}

   void incTargettingRange() { targetting_range += 16; }

   void setFriction(float friction_x, float friction_y)
    {
	 this->friction_x = friction_x;
	 this->friction_y = friction_y;
	}

   void setAcceleration(float acceleration_x, float acceleration_y)
    {
	 this->acceleration_x = acceleration_x;
	 this->acceleration_y = acceleration_y;
	}

   void setPower(float power)                       { this->power = power;                         }

   void setBulletFrame(int bullet_frame)            { this->bullet_frame = bullet_frame;           }

   void setBulletDamage(int bullet_damage)          { this->bullet_damage = bullet_damage;         }

   void setReloadTime(int reload_time)              { this->reload_time = reload_time;             }
   
   void setRotationSpeed(float rotation_speed)      { this->rotation_speed = rotation_speed;       }
   
   void setHullFrame(int hull_frame)                { this->hull_frame = hull_frame;               }
   
   void setAimFrame(int aim_frame)                  { this->aim_frame = aim_frame;                 }

   void setEnableTargetting(bool enable_targetting) { this->enable_targetting = enable_targetting; }

   void setWeapon(int weapon)                       { this->weapon = weapon;                       }

   void setReloadCount(int reload_count)            { this->reload_count = reload_count;           }

   void setSampleShoot(AR_Sample *sample_shoot)     { this->sample_shoot = sample_shoot;           }

   void setScrollXYP(float *scroll_x, float *scroll_y)
    {
	 setScrollXP(scroll_x); setScrollYP(scroll_y);
	 for(int i=0;i<total_bullets;i++)
	  {
	   bullets[i]->setScrollXP(scroll_x);
	   bullets[i]->setScrollYP(scroll_y);
      }
	}

   float getTargettingRangeScale() { return (float)(targetting_range) / 32.0; }

   int getBulletsFired()         { return bullets_fired;     }
   
   int getBulletDamage()         { return bullet_damage;     }

   int getTotalBullets()         { return total_bullets;     }
   
   int getWeapon()               { return weapon;            }

   int getReloadTime()           { return reload_time;       }
   
   int getReloadCount()          { return reload_count;      }

   bool getEnableTargetting()    { return enable_targetting; }
   
   float getRotationSpeed()      { return rotation_speed;    }

   void rotateLeft()  { angle = angle - rotation_speed; }
   
   void rotateRight() { angle = angle + rotation_speed; }

   Bullet **getBulletsP() { return bullets; }

 };
